@@ -1,10 +1,10 @@
 /*
-    Argument
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    argument
+    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,19 +13,16 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #ifndef __ARGUMENT_H__
 #define __ARGUMENT_H__
 
 
-#include "common.h"
+#include "gcommon.h"
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 
 #define ARGUMENT_TYPE_BOOL 0
@@ -50,16 +47,16 @@ typedef struct _ArgumentArray
 {
   gsize leng;
   gint *array;
-  gint *def;
+  const gint *def;
 } ArgumentArray;
 typedef struct _ArgumentString
 {
   gchar *str;
-  gchar *def;
+  const gchar *def;
 } ArgumentString;
 typedef struct _ArgumentExit
 {
-  gchar *str;
+  const gchar *str;
 } ArgumentExit, ArgumentHelp;
 typedef struct _ArgumentList
 {
@@ -86,9 +83,7 @@ arg_analyse (gchar              **argv,
              const ArgumentList  *arg_opts);
 
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 
 #endif /* __ARGUMENT_H__ */
