@@ -1,10 +1,10 @@
 /*
-    Profile
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    profile
+    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,19 +13,18 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "fileio.h"
 #include "misc.h"
 #include "profile.h"
 
 
-#define PROFILE_DATA_TYPE_UNKNOW 0
-#define PROFILE_DATA_TYPE_SPACE 1
+#define PROFILE_DATA_TYPE_UNKNOW  0
+#define PROFILE_DATA_TYPE_SPACE   1
 #define PROFILE_DATA_TYPE_COMMENT 2
 #define PROFILE_DATA_TYPE_SECTION 3
-#define PROFILE_DATA_TYPE_KEY 4
+#define PROFILE_DATA_TYPE_KEY     4
 
 
 typedef struct _ProfileList
@@ -44,12 +43,12 @@ struct _Profile
 
 /******************************************************************************
 *                                                                             *
-* ja:初期化ファイル関数群(内部)                                               *
+* ja:t@C֐Q()                                               *
 *                                                                             *
 ******************************************************************************/
-/*  ja:リストをテキスト化する
-    profile_list,リスト
-             RET,テキストリスト                                             */
+/*  ja:XgeLXg
+    profile_list,Xg
+             RET,eLXgXg                                             */
 static gchar *
 profile_list_to_text (ProfileList *profile_list)
 {
@@ -73,36 +72,47 @@ profile_list_to_text (ProfileList *profile_list)
 }
 
 
-/*  ja:テキストをリスト化する
-    text,テキスト
-     RET,リスト                                                             */
+/*  ja:eLXgXg
+    text,eLXg
+     RET,Xg                                                             */
 static ProfileList *
 profile_list_from_text (const gchar *text)
 {
-  gchar **buf, *section = NULL;
-  gint i;
+  gchar *buf, **str, *section = NULL;
+  gint i, j = 0;
   ProfileList *p = NULL, *q, *r;
 
-  buf = g_strsplit (text, "\n", 0);
-  for (i = 0; buf[i]; i++)
+  /* ja:CRLFLFɕϊ */
+  buf = g_strdup (text);
+  for (i = 0; buf[i] != '\0'; i++)
+    if (buf[i] != '\r' || buf[i + 1] != '\n')
+      buf[j++] = buf[i];
+  buf[j] = '\0';
+  /* ja:CRLFɕϊ */
+  for (i = 0; buf[i] != '\0'; i++)
+    if (buf[i] == '\r')
+      buf[i] = '\n';
+  str = g_strsplit (buf, "\n", 0);
+  g_free (buf);
+  for (i = 0; str[i]; i++)
     {
       q = g_malloc0 (sizeof (ProfileList));
-      q->data = g_strdup (buf[i]);
+      q->data = g_strdup (str[i]);
       q->prev = p;
       if (p)
         p->next = q;
       p = q;
     }
-  g_strfreev (buf);
+  g_strfreev (str);
   if (p)
     while (p->prev)
       p = p->prev;
   r = p;
-  /* ja:リストを整理する */
+  /* ja:Xg𐮗 */
   while (p)
     {
       gsize n;
-      gchar *data;
+      gchar *data, *eq;
 
       data = p->data ? g_strstrip (g_strdup (p->data)) : NULL;
       n = data ? g_strlen (data) : 0;
@@ -118,8 +128,7 @@ profile_list_from_text (const gchar *text)
         {
           p->type = PROFILE_DATA_TYPE_SECTION;
           g_free (section);
-          section = g_strdup (data + 1);
-          section[n - 2] = '\0';
+          section = g_strndup (data + 1, n - 2);
           for (q = p->prev; q && q->type == PROFILE_DATA_TYPE_SPACE;
                                                                 q = q->prev)
             {
@@ -127,19 +136,18 @@ profile_list_from_text (const gchar *text)
               q->section = NULL;
             }
         }
-      else if (g_strchr (data, '='))
+      else if ((eq = g_strchr (p->data, '=')))
         {
           p->type = PROFILE_DATA_TYPE_KEY;
-          p->key = g_strdup (data);
-          *g_strchr (p->key, '=') = '\0';
-          p->value = g_strchr (p->data, '=') + 1;
+          p->key = g_strndup (p->data, eq - p->data);
+          p->value = eq + 1;
         }
       else
         {
           p->type = PROFILE_DATA_TYPE_UNKNOW;
         }
-      p->section = g_strdup (section);
       g_free (data);
+      p->section = g_strdup (section);
       p = p->next;
     }
   g_free (section);
@@ -147,14 +155,14 @@ profile_list_from_text (const gchar *text)
 }
 
 
-/*  ja:ファイルをリスト化する
-    file,ファイル名
-     RET,リスト                                                             */
+/*  ja:t@CXg
+    file,t@C
+     RET,Xg                                                             */
 static ProfileList *
 profile_list_from_file (const gchar *file)
 {
   gchar *data;
-  gsize length;
+  gssize length;
   ProfileList *p = NULL;
 
   data = fileio_load (file, &length);
@@ -169,8 +177,8 @@ profile_list_from_file (const gchar *file)
 }
 
 
-/*  ja:初期化ファイルを解放する
-    profile_list,リスト                                                     */
+/*  ja:t@C
+    profile_list,Xg                                                     */
 static void
 profile_list_free (ProfileList *profile_list)
 {
@@ -189,12 +197,12 @@ profile_list_free (ProfileList *profile_list)
 
 /******************************************************************************
 *                                                                             *
-* ja:初期化ファイル関数群(独自)                                               *
+* ja:t@C֐Q(Ǝ)                                               *
 *                                                                             *
 ******************************************************************************/
-/*  ja:初期化ファイルを開く
-    file,ファイル名
-     RET,プロファイル                                                       */
+/*  ja:t@CJ
+    file,t@C
+     RET,vt@C                                                       */
 Profile *
 profile_open (const gchar *file)
 {
@@ -259,9 +267,9 @@ profile_open (const gchar *file)
 }
 
 
-/*  ja:初期化ファイルを閉じる
-    profile,プロファイル
-        RET,TRUE:正常終了,FALSE:エラー                                      */
+/*  ja:t@C
+    profile,vt@C
+        RET,TRUE:I,FALSE:G[                                      */
 gboolean
 profile_close (Profile *profile)
 {
@@ -269,7 +277,7 @@ profile_close (Profile *profile)
     return TRUE;
   if (profile->edit)
     {
-      /* ja:変更があったとき */
+      /* ja:ύXƂ */
       gchar *text;
 
       text = profile_list_to_text (profile->list);
@@ -287,11 +295,11 @@ profile_close (Profile *profile)
 }
 
 
-/*  ja:初期化ファイルから文字列を取得する
-    profile,プロファイル
-    section,セクション
-        key,キー
-        RET,文字列,NULL:エラー                                              */
+/*  ja:t@C當擾
+    profile,vt@C
+    section,ZNV
+        key,L[
+        RET,,NULL:G[                                              */
 gchar *
 profile_get_string (Profile     *profile,
                     const gchar *section,
@@ -313,12 +321,12 @@ profile_get_string (Profile     *profile,
 }
 
 
-/*  ja:初期化ファイルから値のサイズ取得する
-    profile,プロファイル
-    section,セクション
-        key,キー
-       type,タイプ
-        RET,バイト数,0:エラー                                               */
+/*  ja:t@Cl̃TCY擾
+    profile,vt@C
+    section,ZNV
+        key,L[
+       type,^Cv
+        RET,oCg,0:G[                                               */
 gsize
 profile_get_size (Profile     *profile,
                   const gchar *section,
@@ -369,14 +377,14 @@ profile_get_size (Profile     *profile,
 }
 
 
-/*  ja:初期化ファイルから値を取得する
-    profile,プロファイル
-    section,セクション
-        key,キー
-      value,値を入れるバッファ
-       size,値を入れるバッファのサイズ
-       type,タイプ
-        RET,TRUE:正常終了,FALSE:エラー                                      */
+/*  ja:t@Cl擾
+    profile,vt@C
+    section,ZNV
+        key,L[
+      value,lobt@
+       size,lobt@̃TCY
+       type,^Cv
+        RET,TRUE:I,FALSE:G[                                      */
 gboolean
 profile_get_value (Profile     *profile,
                    const gchar *section,
@@ -386,7 +394,7 @@ profile_get_value (Profile     *profile,
                    const guint  type)
 {
   guint8 *array;
-  gint n;
+  gsize n;
   ProfileList *p;
 
   if (!profile || !section || !key || !value)
@@ -447,14 +455,14 @@ profile_get_value (Profile     *profile,
 }
 
 
-/*  ja:初期化ファイルに値を設定する
-    profile,プロファイル
-    section,セクション
-        key,キー
-      value,値が入っているバッファ
-       size,値が入っているバッファのサイズ
-       type,タイプ
-        RET,TRUE:正常終了,FALSE:エラー                                      */
+/*  ja:t@Cɒlݒ肷
+    profile,vt@C
+    section,ZNV
+        key,L[
+      value,lĂobt@
+       size,lĂobt@̃TCY
+       type,^Cv
+        RET,TRUE:I,FALSE:G[                                      */
 gboolean
 profile_set_value (Profile       *profile,
                    const gchar   *section,
@@ -482,7 +490,7 @@ profile_set_value (Profile       *profile,
         {
           if (q)
             {
-              /* ja:前のデータとの間にスペースを入れる */
+              /* ja:Õf[^Ƃ̊ԂɃXy[X */
               p = g_malloc (sizeof (ProfileList));
               p->type = PROFILE_DATA_TYPE_SPACE;
               p->data = p->section = p->key = p->value = NULL;
@@ -491,7 +499,7 @@ profile_set_value (Profile       *profile,
               q->next = p;
               q = p;
             }
-          /* ja:セクションもキーもないときにはセクションを作る */
+          /* ja:ZNVL[ȂƂɂ̓ZNV */
           p = g_malloc (sizeof (ProfileList));
           p->type = PROFILE_DATA_TYPE_SECTION;
           p->data = g_strdup_printf ("[%s]", section);
@@ -513,7 +521,7 @@ profile_set_value (Profile       *profile,
       while (q->type == PROFILE_DATA_TYPE_SPACE && q->section
                             && g_strcmp (p->section, section) == 0 && q->prev)
         q = q->prev;
-      /* ja:セクションの最後にキーを作る */
+      /* ja:ZNV̍ŌɃL[ */
       p = g_malloc (sizeof (ProfileList));
       p->type = PROFILE_DATA_TYPE_KEY;
       p->data = g_strdup_printf ("%s=", key);
@@ -561,10 +569,10 @@ profile_set_value (Profile       *profile,
 }
 
 
-/*  ja:初期化ファイルのセクションを削除する
-    profile,プロファイル
-    section,セクション
-        RET,TRUE:正常終了,FALSE:エラー                                      */
+/*  ja:t@C̃ZNV폜
+    profile,vt@C
+    section,ZNV
+        RET,TRUE:I,FALSE:G[                                      */
 gboolean
 profile_delete_section (Profile     *profile,
                         const gchar *section)
@@ -595,11 +603,11 @@ profile_delete_section (Profile     *profile,
 }
 
 
-/*  ja:初期化ファイルのキーを削除する
-    profile,プロファイル
-    section,セクション
-        key,キー
-        RET,TRUE:正常終了,FALSE:エラー                                      */
+/*  ja:t@C̃L[폜
+    profile,vt@C
+    section,ZNV
+        key,L[
+        RET,TRUE:I,FALSE:G[                                      */
 gboolean
 profile_delete_key (Profile     *profile,
                     const gchar *section,
@@ -632,9 +640,9 @@ profile_delete_key (Profile     *profile,
 }
 
 
-/*  ja:初期化ファイルのセクションを列挙する
-    profile,プロファイル
-        RET,セクションのリスト,NULL:エラー                                  */
+/*  ja:t@C̃ZNV񋓂
+    profile,vt@C
+        RET,ZNṼXg,NULL:G[                                  */
 GList *
 profile_enum_section (Profile *profile)
 {
@@ -655,10 +663,10 @@ profile_enum_section (Profile *profile)
 }
 
 
-/*  ja:初期化ファイルのキーを列挙する
-    profile,プロファイル
-    section,セクション
-        RET,セクションのリスト,NULL:エラー                                  */
+/*  ja:t@C̃L[񋓂
+    profile,vt@C
+    section,ZNV
+        RET,ZNṼXg,NULL:G[                                  */
 GList *
 profile_enum_key (Profile     *profile,
                   const gchar *section)
@@ -684,48 +692,47 @@ profile_enum_key (Profile     *profile,
 
 /******************************************************************************
 *                                                                             *
-* ja:初期化ファイル関数群(互換)                                               *
+* ja:t@C֐Q(݊)                                               *
 *                                                                             *
 ******************************************************************************/
-#if ! GLIB_CHECK_VERSION(2,6,0)
-GKeyFile *
-g_key_file_new (void)
+Profile *
+profile_keyfile_new (void)
 {
   return g_malloc0 (sizeof (Profile));
 }
 
 
 void
-g_key_file_free (GKeyFile *key_file)
+profile_keyfile_free (Profile *profile)
 {
-  if (key_file)
+  if (profile)
     {
-      profile_list_free (key_file->list);
-      profile_list_free (key_file->sublist);
-      g_free (key_file->file);
-      g_free (key_file);
+      profile_list_free (profile->list);
+      profile_list_free (profile->sublist);
+      g_free (profile->file);
+      g_free (profile);
     }
 }
 
 
 gboolean
-g_key_file_load_from_file (GKeyFile       *key_file,
-                           const gchar    *file,
-                           GKeyFileFlags   flags,
-                           GError        **error)
+profile_keyfile_load_from_file (Profile      *profile,
+                                const gchar  *file,
+                                guint         flags,
+                                GError      **error)
 {
   gboolean result = FALSE;
 
   if (error)
     *error = NULL;
-  if (key_file)
+  if (profile)
     {
       gchar *data;
-      gsize length;
+      gssize length;
 
       data = fileio_load (file, &length);
       if (data)
-        result = g_key_file_load_from_data (key_file, data, length,
+        result = profile_keyfile_load_from_data (profile, data, length,
                                                                 flags, error);
       g_free (data);
     }
@@ -734,42 +741,49 @@ g_key_file_load_from_file (GKeyFile       *key_file,
 
 
 gboolean
-g_key_file_load_from_data (GKeyFile       *key_file,
-                           const gchar    *data,
-                           gsize           length,
-                           GKeyFileFlags   flags,
-                           GError        **error)
+profile_keyfile_load_from_data (Profile      *profile,
+                                const gchar  *data,
+                                gsize         length,
+                                guint         flags,
+                                GError      **error)
 {
   if (error)
     *error = NULL;
-  if (!key_file)
+  if (!profile)
     return FALSE;
-  /* ja:リスト解放 */
-  profile_list_free (key_file->list);
-  profile_list_free (key_file->sublist);
-  key_file->list = key_file->sublist = NULL;
+  /* ja:Xg */
+  profile_list_free (profile->list);
+  profile_list_free (profile->sublist);
+  profile->list = profile->sublist = NULL;
   if (data && length != 0)
     {
-      gchar *text;
+      if (length == (gsize)-1)
+        {
+          gchar *text;
 
-      text = length == (gsize)-1 ? g_strdup (data) : g_strndup (data, length);
-      key_file->list = profile_list_from_text (text);
-      g_free (text);
+          text = g_strndup (data, length);
+          profile->list = profile_list_from_text (text);
+          g_free (text);
+        }
+      else
+        {
+          profile->list = profile_list_from_text (data);
+        }
     }
   return TRUE;
 }
 
 
 gchar *
-g_key_file_to_data (GKeyFile  *key_file,
-                    gsize     *length,
-                    GError   **error)
+profile_keyfile_to_data (Profile  *profile,
+                         gsize    *length,
+                         GError  **error)
 {
   gchar *data;
 
   if (error)
     *error = NULL;
-  data = key_file ? profile_list_to_text (key_file->list) : NULL;
+  data = profile ? profile_list_to_text (profile->list) : NULL;
   if (data && length)
     *length = g_strlen (data);
   return data;
@@ -777,13 +791,13 @@ g_key_file_to_data (GKeyFile  *key_file,
 
 
 gchar **
-g_key_file_get_groups (GKeyFile *key_file,
-                       gsize    *length)
+profile_keyfile_get_groups (Profile *profile,
+                            gsize   *length)
 {
   gchar **str = NULL;
   GList *glist;
 
-  glist = profile_enum_section (key_file);
+  glist = profile_enum_section (profile);
   if (glist)
     {
       guint i, leng;
@@ -802,17 +816,17 @@ g_key_file_get_groups (GKeyFile *key_file,
 
 
 gchar **
-g_key_file_get_keys (GKeyFile     *key_file,
-                     const gchar  *group_name,
-                     gsize        *length,
-                     GError      **error)
+profile_keyfile_get_keys (Profile      *profile,
+                          const gchar  *section,
+                          gsize        *length,
+                          GError      **error)
 {
   gchar **str = NULL;
   GList *glist;
 
   if (error)
     *error = NULL;
-  glist = profile_enum_key (key_file, group_name);
+  glist = profile_enum_key (profile, section);
   if (glist)
     {
       guint i, leng;
@@ -831,18 +845,18 @@ g_key_file_get_keys (GKeyFile     *key_file,
 
 
 gboolean
-g_key_file_has_group (GKeyFile    *key_file,
-                      const gchar *group_name)
+profile_keyfile_has_group (Profile     *profile,
+                           const gchar *section)
 {
-  if (key_file && group_name)
+  if (profile && section)
     {
       ProfileList *p;
 
-      for (p = key_file->list; p; p = p->next)
-        if (p->section && g_strcmp (p->section, group_name) == 0)
+      for (p = profile->list; p; p = p->next)
+        if (p->section && g_strcmp (p->section, section) == 0)
           return TRUE;
-      for (p = key_file->sublist; p; p = p->next)
-        if (p->section && g_strcmp (p->section, group_name) == 0)
+      for (p = profile->sublist; p; p = p->next)
+        if (p->section && g_strcmp (p->section, section) == 0)
           return TRUE;
     }
   return FALSE;
@@ -850,23 +864,23 @@ g_key_file_has_group (GKeyFile    *key_file,
 
 
 gboolean
-g_key_file_has_key (GKeyFile     *key_file,
-                    const gchar  *group_name,
-                    const gchar  *key,
-                    GError      **error)
+profile_keyfile_has_key (Profile      *profile,
+                         const gchar  *section,
+                         const gchar  *key,
+                         GError      **error)
 {
   if (error)
     *error = NULL;
-  if (key_file && group_name && key)
+  if (profile && section && key)
     {
       ProfileList *p;
 
-      for (p = key_file->list; p; p = p->next)
-        if (p->section && p->key && g_strcmp (p->section, group_name) == 0
+      for (p = profile->list; p; p = p->next)
+        if (p->section && p->key && g_strcmp (p->section, section) == 0
                                                 && g_strcmp (p->key, key) == 0)
           return TRUE;
-      for (p = key_file->sublist; p; p = p->next)
-        if (p->section && p->key && g_strcmp (p->section, group_name) == 0
+      for (p = profile->sublist; p; p = p->next)
+        if (p->section && p->key && g_strcmp (p->section, section) == 0
                                                 && g_strcmp (p->key, key) == 0)
           return TRUE;
     }
@@ -875,28 +889,112 @@ g_key_file_has_key (GKeyFile     *key_file,
 
 
 gchar *
-g_key_file_get_string (GKeyFile     *key_file,
-                       const gchar  *group_name,
-                       const gchar  *key,
-                       GError      **error)
+profile_keyfile_get_value (Profile      *profile,
+                           const gchar  *section,
+                           const gchar  *key,
+                           GError      **error)
 {
   if (error)
     *error = NULL;
-  return profile_get_string (key_file, group_name, key);
+  return profile_get_string (profile, section, key);
+}
+
+
+gchar *
+profile_keyfile_get_string (Profile      *profile,
+                            const gchar  *section,
+                            const gchar  *key,
+                            GError      **error)
+{
+  gchar *str, *ret;
+  gint i = 0, j = 0;
+
+  if (error)
+    *error = NULL;
+  str = profile_get_string (profile, section, key);
+  if (!str)
+    return NULL;
+  ret = g_malloc ((g_strlen (str) + 1) * sizeof (gchar));
+  while (str[i] != '\0')
+    if (str[i] == '\\' && str[i + 1] != '\0')
+      switch (str[++i])
+        {
+          case 'a': ret[j++] = '\a'; i++; break;
+          case 'b': ret[j++] = '\b'; i++; break;
+          case 't': ret[j++] = '\t'; i++; break;
+          case 'n': ret[j++] = '\n'; i++; break;
+          case 'v': ret[j++] = '\v'; i++; break;
+          case 'f': ret[j++] = '\f'; i++; break;
+          case 'r': ret[j++] = '\r'; i++; break;
+          case 's': ret[j++] = ' ';  i++; break;
+          case 'u':
+          case 'x':
+          case 'U':
+            {
+              gunichar c = 0;
+              gint k, len;
+
+              len = str[i] == 'x' ? 2 : str[i] == 'u' ? 4 : 8;
+              for (k = i + 1; k <= i + len; k++)
+                {
+                  gint n;
+
+                  n = g_ascii_isxdigit (str[k]);
+                  if (n < 0)
+                    break;
+                  c = c * 16 + n;
+                }
+              if (k > i + len)
+                {
+                  gchar utf8str[6];
+
+                  i += len + 1;
+                  len = g_unichar_to_utf8 (c, utf8str);
+                  for (k = 0; k < len; k++)
+                    ret[j++] = utf8str[k];
+                }
+              else
+                {
+                  ret[j++] = str[i++];
+                }
+            }
+            break;
+          default:
+            if ('0' <= str[i] && str[i] <= '7'
+                                    && '0' <= str[i + 1] && str[i + 1] <= '7'
+                                    && '0' <= str[i + 2] && str[i + 2] <= '7')
+              {
+                gint k;
+
+                ret[j] = 0;
+                for (k = 0; k < 3; k++)
+                  ret[j] = ret[j] * 8 + g_ascii_digit_value (str[i++]);
+                j++;
+              }
+            else
+              {
+                ret[j++] = str[i++];
+              }
+        }
+    else
+      ret[j++] = str[i++];
+  ret[j] = '\0';
+  g_free (str);
+  return ret;
 }
 
 
 gboolean
-g_key_file_get_boolean (GKeyFile     *key_file,
-                        const gchar  *group_name,
-                        const gchar  *key,
-                        GError      **error)
+profile_keyfile_get_boolean (Profile      *profile,
+                             const gchar  *section,
+                             const gchar  *key,
+                             GError      **error)
 {
   gboolean value;
 
   if (error)
     *error = NULL;
-  if (!profile_get_value (key_file, group_name, key,
+  if (!profile_get_value (profile, section, key,
                 (gpointer)&value, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL))
     value = FALSE;
   return value;
@@ -904,16 +1002,16 @@ g_key_file_get_boolean (GKeyFile     *key_file,
 
 
 gint
-g_key_file_get_integer (GKeyFile     *key_file,
-                        const gchar  *group_name,
-                        const gchar  *key,
-                        GError      **error)
+profile_keyfile_get_integer (Profile      *profile,
+                             const gchar  *section,
+                             const gchar  *key,
+                             GError      **error)
 {
   gint value;
 
   if (error)
     *error = NULL;
-  if (!profile_get_value (key_file, group_name, key,
+  if (!profile_get_value (profile, section, key,
                     (gpointer)&value, sizeof (gint), PROFILE_VALUE_TYPE_INT))
     value = 0;
   return value;
@@ -921,23 +1019,22 @@ g_key_file_get_integer (GKeyFile     *key_file,
 
 
 gint *
-g_key_file_get_integer_list (GKeyFile     *key_file,
-                             const gchar  *group_name,
-                             const gchar  *key,
-                             gsize        *length,
-                             GError      **error)
+profile_keyfile_get_integer_list (Profile      *profile,
+                                  const gchar  *section,
+                                  const gchar  *key,
+                                  gsize        *length,
+                                  GError      **error)
 {
   gsize leng;
   gint *value = NULL;
 
   if (error)
     *error = NULL;
-  leng = profile_get_size (key_file, group_name, key,
-                                                    PROFILE_VALUE_TYPE_ARRAY);
+  leng = profile_get_size (profile, section, key, PROFILE_VALUE_TYPE_ARRAY);
   if (leng > 0)
     {
       value = g_malloc (leng);
-      if (!profile_get_value (key_file, group_name, key,
+      if (!profile_get_value (profile, section, key,
                             (gpointer)value, leng, PROFILE_VALUE_TYPE_ARRAY))
         {
           g_free (value);
@@ -952,69 +1049,126 @@ g_key_file_get_integer_list (GKeyFile     *key_file,
 
 
 void
-g_key_file_set_string (GKeyFile    *key_file,
-                       const gchar *group_name,
-                       const gchar *key,
-                       const gchar *string)
+profile_keyfile_set_value (Profile     *profile,
+                           const gchar *section,
+                           const gchar *key,
+                           const gchar *string)
 {
-  profile_set_value (key_file, group_name, key, (gpointer)string,
+  profile_set_value (profile, section, key, (gpointer)string,
         (g_strlen (string) + 1) * sizeof (gchar), PROFILE_VALUE_TYPE_STRING);
 }
 
 
 void
-g_key_file_set_boolean (GKeyFile    *key_file,
-                        const gchar *group_name,
-                        const gchar *key,
-                        gboolean     value)
+profile_keyfile_set_string (Profile     *profile,
+                            const gchar *section,
+                            const gchar *key,
+                            const gchar *string)
 {
-  profile_set_value (key_file, group_name, key,
+  gchar *str;
+  gint i = 0;
+
+  str = g_malloc ((g_strlen (string) * 4 + 1) * sizeof (gchar));
+  str[0] = '\0';
+  while (string[i] != '\0')
+    {
+      gchar tmp[7];
+
+      switch (string[i])
+        {
+          case '\a': tmp[0] = '\\'; tmp[1] = 'a';  tmp[2] = '\0'; i++; break;
+          case '\b': tmp[0] = '\\'; tmp[1] = 'b';  tmp[2] = '\0'; i++; break;
+          case '\t': tmp[0] = '\\'; tmp[1] = 't';  tmp[2] = '\0'; i++; break;
+          case '\n': tmp[0] = '\\'; tmp[1] = 'n';  tmp[2] = '\0'; i++; break;
+          case '\v': tmp[0] = '\\'; tmp[1] = 'v';  tmp[2] = '\0'; i++; break;
+          case '\f': tmp[0] = '\\'; tmp[1] = 'f';  tmp[2] = '\0'; i++; break;
+          case '\r': tmp[0] = '\\'; tmp[1] = 'r';  tmp[2] = '\0'; i++; break;
+          case '\\': tmp[0] = '\\'; tmp[1] = '\\'; tmp[2] = '\0'; i++; break;
+          default:
+            {
+              gunichar c;
+
+              c = g_utf8_get_char_validated (string + i, -1);
+              if (c == (gunichar)-1 || c == (gunichar)-2
+                                                    || !g_unichar_isprint (c))
+                {
+                  tmp[0] = '\\';
+                  tmp[1] = '0' + (((string[i]) >> 6) & 07);
+                  tmp[2] = '0' + (((string[i]) >> 3) & 07);
+                  tmp[3] = '0' + ((string[i]) & 07);
+                  tmp[4] = '\0';
+                  i++;
+                }
+              else
+                {
+                  gint n;
+
+                  n = g_unichar_to_utf8 (c, tmp);
+                  tmp[n] = '\0';
+                  i += n;
+                }
+            }
+        }
+      g_strcat (str, tmp);
+    }
+  profile_set_value (profile, section, key, (gpointer)str,
+            (g_strlen (str) + 1) * sizeof (gchar), PROFILE_VALUE_TYPE_STRING);
+  g_free (str);
+}
+
+
+void
+profile_keyfile_set_boolean (Profile     *profile,
+                             const gchar *section,
+                             const gchar *key,
+                             gboolean     value)
+{
+  profile_set_value (profile, section, key,
                 (gpointer)&value, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
 }
 
 
 void
-g_key_file_set_integer (GKeyFile    *key_file,
-                        const gchar *group_name,
-                        const gchar *key,
-                        gint         value)
+profile_keyfile_set_integer (Profile     *profile,
+                             const gchar *section,
+                             const gchar *key,
+                             gint         value)
 {
-  profile_set_value (key_file, group_name, key,
+  profile_set_value (profile, section, key,
                     (gpointer)&value, sizeof (gint), PROFILE_VALUE_TYPE_INT);
 }
 
 
 void
-g_key_file_set_integer_list (GKeyFile    *key_file,
-                             const gchar *group_name,
-                             const gchar *key,
-                             gint         list[],
-                             gsize        length)
+profile_keyfile_set_integer_list (Profile     *profile,
+                                  const gchar *section,
+                                  const gchar *key,
+                                  gint         list[],
+                                  gsize        length)
 {
-  profile_set_value (key_file, group_name, key,
+  profile_set_value (profile, section, key,
             (gpointer)list, length * sizeof (gint), PROFILE_VALUE_TYPE_ARRAY);
 }
 
 
 void
-g_key_file_remove_group (GKeyFile     *key_file,
-                         const gchar  *group_name,
-                         GError      **error)
+profile_keyfile_remove_group (Profile      *profile,
+                              const gchar  *section,
+                              GError      **error)
 {
   if (error)
     *error = NULL;
-  profile_delete_section (key_file, group_name);
+  profile_delete_section (profile, section);
 }
 
 
 void
-g_key_file_remove_key (GKeyFile     *key_file,
-                       const gchar  *group_name,
-                       const gchar  *key,
-                       GError      **error)
+profile_keyfile_remove_key (Profile      *profile,
+                            const gchar  *section,
+                            const gchar  *key,
+                            GError      **error)
 {
   if (error)
     *error = NULL;
-  profile_delete_key (key_file, group_name, key);
+  profile_delete_key (profile, section, key);
 }
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
