@@ -17,6 +17,7 @@
 */
 #include "orzmdi.h"
 #include "misc/fileio.h"
+#include "misc/misc.h"
 
 
 enum
@@ -533,8 +534,7 @@ orz_mdi_create_tab_label (OrzMdi         *mdi,
                           gpointer        page_data)
 {
   GdkPixbuf *pixbuf;
-  GtkIconSet *icon_set;
-  GtkWidget *image = NULL, *button, *hbox, *event;
+  GtkWidget *button, *hbox, *event;
   GtkTargetEntry dnd_entries[1] = {{"ORZ_MDI_TAB",
                                     GTK_TARGET_SAME_APP, TARGET_ORZ_MDI_TAB}};
   OrzMdiTab *tab;
@@ -550,108 +550,9 @@ orz_mdi_create_tab_label (OrzMdi         *mdi,
   tab->mdi = mdi;
   tab->child = child;
   file_add_edit (mdi, tab);
-  /* ja:イメージ */
-  icon_set = gtk_icon_factory_lookup_default (GTK_STOCK_CLOSE);
-  if (icon_set)
-    {
-      pixbuf = gtk_icon_set_render_icon (icon_set,
-                                         gtk_widget_get_default_style (),
-                                         GTK_TEXT_DIR_NONE,
-                                         GTK_STATE_NORMAL,
-                                         GTK_ICON_SIZE_MENU,
-                                         NULL, NULL);
-      if (pixbuf)
-        {
-          if (gdk_pixbuf_get_colorspace (pixbuf) == GDK_COLORSPACE_RGB
-                            && gdk_pixbuf_get_n_channels (pixbuf) == 4
-                            && gdk_pixbuf_get_has_alpha (pixbuf)
-                            && gdk_pixbuf_get_bits_per_sample (pixbuf) == 8)
-            {
-              int width, height, rowstride;
-              gint x, y, left, top, right, bottom;
-              guchar *pixels;
-
-              pixels = gdk_pixbuf_get_pixels (pixbuf);
-              width = gdk_pixbuf_get_width (pixbuf);
-              height = gdk_pixbuf_get_height (pixbuf);
-              rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-              for (x = 0; x < width; x++)
-                for (y = 0; y < height; y++)
-                  if (pixels[y * rowstride + x * 4 + 3] > 0)
-                    goto loop_left;
-              loop_left: left = x;
-              for (y = 0; y < height; y++)
-                for (x = 0; x < width; x++)
-                  if (pixels[y * rowstride + x * 4 + 3] > 0)
-                    goto loop_top;
-              loop_top: top = y;
-              for (x = width - 1; x >= 0; x--)
-                for (y = 0; y < height; y++)
-                  if (pixels[y * rowstride + x * 4 + 3] > 0)
-                    goto loop_right;
-              loop_right: right = x;
-              for (y = height - 1; y >= 0; y--)
-                for (x = 0; x < width; x++)
-                  if (pixels[y * rowstride + x * 4 + 3] > 0)
-                    goto loop_bottom;
-              loop_bottom: bottom = y;
-              if (left <= right && top <= bottom)
-                {
-                  int w, h, r;
-                  guchar *p;
-                  gint i;
-                  GdkPixbuf *b;
-
-                  w = right - left + 1;
-                  h = bottom - top + 1;
-                  b = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w, h);
-                  p = gdk_pixbuf_get_pixels (b);
-                  r = gdk_pixbuf_get_rowstride (b);
-                  for (x = 0; x < w; x++)
-                    for (y = 0; y < h; y++)
-                      for (i = 0; i < 4; i++)
-                        p[y * r + x * 4 + i]
-                        = pixels[(y + top) * rowstride + (x + left) * 4 + i];
-                  image = gtk_image_new_from_pixbuf (b);
-                  g_object_unref (b);
-                }
-            }
-          g_object_unref (pixbuf);
-        }
-    }
-  if (!image)
-    image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-  if (!image)
-    {
-      const static gchar *xpm[] = {
-"12 13 4 1",
-"   c None",
-"X  c #000000",
-"+  c #808080",
-".  c #FFFFFF",
-"+++++++++++.",
-"+.........+.",
-"+.        +.",
-"+.XX    XX+.",
-"+. XX  XX +.",
-"+.  XXXX  +.",
-"+.   XX   +.",
-"+.  XXXX  +.",
-"+. XX  XX +.",
-"+.XX    XX+.",
-"+.        +.",
-"+++++++++++.",
-"............"};
-
-      pixbuf = gdk_pixbuf_new_from_xpm_data (xpm);
-      image = gtk_image_new_from_pixbuf (pixbuf);
-      g_object_unref (pixbuf);
-    }
   /* ja:ボタン */
-  button = gtk_button_new ();
-  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-  gtk_container_add (GTK_CONTAINER (button), image);
-  gtk_widget_show_all (button);
+  button = misc_close_button ();
+  gtk_widget_show (button);
   /* ja:ラベル */
   tab->label = gtk_label_new ("");
   orz_mdi_label_markup (tab);
