@@ -25,18 +25,12 @@
 G_BEGIN_DECLS
 
 
-typedef struct _FindString
-{
-  gboolean arrow;
-  gboolean ignorecase;
-  gchar *text;
-} FindString;
+#define FIND_TEXT 16
 
 
 extern gboolean find_arrow;                 /* ja:検索の設定 */
 extern gboolean find_ignorecase;            /* ja:検索の設定 */
-extern gchar *find_text[32];                /* ja:検索する文字列 */
-extern gint find_num;                       /* ja:登録されている検索文字列 */
+extern gchar *find_text[FIND_TEXT];         /* ja:検索する文字列 */
 
 
 /******************************************************************************
@@ -45,13 +39,14 @@ extern gint find_num;                       /* ja:登録されている検索文
 *                                                                             *
 ******************************************************************************/
 /*  ja:検索
-         tmaid,ウインドウ情報
+         tmaid,TXTウインドウ情報
          start,検索範囲
            end,検索範囲(x=-1ならば選択範囲なし)
           find,検索文字列
          arrow,TRUE:下へ,FALSE:上へ
-    ignorecase,TRUE:大文字小文字を区別する,FALSE:大文字小文字を区別しない   */
-void
+    ignorecase,TRUE:大文字小文字を区別する,FALSE:大文字小文字を区別しない
+           RET,TRUE:文字列あり,FALSE:文字列なし                             */
+gboolean
 find_operation (TmaidWindow    *tmaid,
                 GdkPoint       *start,
                 GdkPoint       *end,
@@ -60,18 +55,6 @@ find_operation (TmaidWindow    *tmaid,
                 const gboolean  ignorecase);
 
 
-/******************************************************************************
-*                                                                             *
-* ja:検索ダイアログ関数群                                                     *
-*                                                                             *
-******************************************************************************/
-/*  ja:検索ダイアログ表示
-     fd,検索情報
-    RET,TRUE:OK,FALSE:Cancel                                                */
-gboolean
-find_dialog (FindString *fd);
-
-
 G_END_DECLS
 
 
